// ClipboardFormats_as.hx:  ActionScript 3 "ClipboardFormats" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090514 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.


// NOTE: this only works with flash v.9 and above

#if flash9 || !flash
import flash.desktop.ClipboardFormats;
import flash.display.MovieClip;
#end
import flash.Lib;
import Type;

// import our testing API
import DejaGnu;

// Class must be named with the _as suffix, as that's the same name as the file.
class ClipboardFormats_as {
    static function main() {
        #if !flash9
		DejaGnu.note("This does not apply to lower than flash v9");
		#end
		
		var x1:ClipboardFormats = new ClipboardFormats();

        // Make sure we actually get a valid class        
        if (x1 != null) {
            DejaGnu.pass("ClipboardFormats class exists");
        } else {
            DejaGnu.fail("ClipboardFormats lass doesn't exist");
        }

// Tests to see if all the methods exist. All these do is test for
// existance of a method, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.
		#if flash9
		if (ClipboardFormats.HTML_FORMAT == "air:html") {
			DejaGnu.pass("ClipboardFormats::HTML_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::HTML_FORMAT string is not right");
		}
		
		if (ClipboardFormats.RICH_TEXT_FORMAT == "air:rtf") {
			DejaGnu.pass("ClipboardFormats::RICH_TEXT_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::RICH_TEXT_FORMAT string is not right");
		}
		
		if (ClipboardFormats.TEXT_FORMAT == "air:text") {
			DejaGnu.pass("ClipboardFormats::TEXT_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::TEXT_FORMAT string is not right");
		}
		#end
		#if !flash
		if (ClipboardFormats.BITMAP_FORMAT == "air:bitmap") {
			DejaGnu.pass("ClipboardFormats::BITMAP_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::BITMAP_FORMAT string is not right");
		}
		if (ClipboardFormats.FILE_LIST_FORMAT == "air:file list") {
			DejaGnu.pass("ClipboardFormats::FILE_LIST_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::FILE_LIST_FORMAT string is not right");
		}
		if (ClipboardFormats.URL_FORMAT == "air:url") {
			DejaGnu.pass("ClipboardFormats::URL_FORMAT properly exists");
		} else {
			DejaGnu.fail("ClipboardFormats::URL_FORMAT string is not right");
		}
		#end
        // Call this after finishing all tests. It prints out the totals.
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

