// FontStyle_as.hx:  ActionScript 3 "FontStyle" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090515 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.

#if flash9
import flash.text.FontStyle;
import flash.display.MovieClip;
#else
//import flash.FontStyle;
//import flash.MovieClip;
#end
import flash.Lib;
import Type;
import Std;

// import our testing API
import DejaGnu;

// Class must be named with the _as suffix, as that's the same name as the file.
class FontStyle_as {
    static function main() {
		
#if !flash9
		DejaGnu.note("FontStyle does not exist in versions prior to flash9");
#else
        
		//FontStyle contains constants used be TextRenderer
		//constants BOLD, BOLD_ITALIC, ITALIC, REGULAR
		if (Std.is(FontStyle.BOLD, String) && (Std.string(FontStyle.BOLD) == "bold")) {
			DejaGnu.pass("FontStyle::BOLD property exists");
		} else {
			DejaGnu.fail("FontStyle::BOLD property does not exist");
		}
		if (Std.is(FontStyle.BOLD_ITALIC, String) && (Std.string(FontStyle.BOLD_ITALIC) == "boldItalic")) {
			DejaGnu.pass("FontStyle::BOLD_ITALIC property exists");
		} else {
			DejaGnu.fail("FontStyle::BOLD_ITALIC property does not exist");
		}
		if (Std.is(FontStyle.ITALIC, String) && (Std.string(FontStyle.ITALIC) == "italic")) {
			DejaGnu.pass("FontStyle::ITALIC property exists");
		} else {
			DejaGnu.fail("FontStyle::ITALIC property does not exist");
		}
		if (Std.is(FontStyle.REGULAR, String) && (Std.string(FontStyle.REGULAR) == "regular")) {
			DejaGnu.pass("FontStyle::REGULAR property exists");
		} else {
			DejaGnu.fail("FontStyle::REGULAR property does not exist");
		}


#end
        // Call this after finishing all tests. It prints out the totals.
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

