// SimpleButton_as.hx:  ActionScript 3 "SimpleButton" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090514 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.

#if flash9
import flash.display.SimpleButton;
import flash.display.DisplayObject;
import flash.display.MovieClip;
import flash.media.SoundTransform;
import flash.display.Shape;
#else
import flash.MovieClip;
#end
import flash.Lib;
import Type;
import Std;

// import our testing API
import DejaGnu;

// Class must be named with the _as suffix, as that's the same name as the file.
class SimpleButton_as {
    static function main() {
	
#if flash9
        var x1:SimpleButton = new SimpleButton();
	var x2:DisplayObject = new Shape();

	x1.downState = x2;
	x1.hitTestState = x2;
	x1.overState = x2;
	x1.upState = x2;
	
        // Make sure we actually get a valid class  
	if (Std.is(x1, SimpleButton)) {      
            DejaGnu.pass("SimpleButton class exists");
        } else {
            DejaGnu.fail("SimpleButton lass doesn't exist");
        }
// Tests to see if all the properties exist. All these do is test for
// existance of a property, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.
	if (Std.is(x1.downState, DisplayObject)) {
 	    DejaGnu.pass("SimpleButton::downState property exists");
 	} else {
 	    DejaGnu.fail("SimpleButton::downState property doesn't exist");
 	}
	if (Std.is(x1.enabled, Bool)) {
	    DejaGnu.pass("SimpleButton::enabled property exists");
	} else {
	    DejaGnu.fail("SimpleButton::enabled property doesn't exist");
	}
	if (Std.is(x1.hitTestState, DisplayObject)) {
 	    DejaGnu.pass("SimpleButton::hitTestState property exists");
 	} else {
 	    DejaGnu.fail("SimpleButton::hitTestState property doesn't exist");
 	}
 	if (Std.is(x1.overState, DisplayObject)) {
 	    DejaGnu.pass("SimpleButton::overState property exists");
 	} else {
 	    DejaGnu.fail("SimpleButton::overState property doesn't exist");
 	}
	if (Std.is(x1.soundTransform, SoundTransform)) {
 	    DejaGnu.pass("SimpleButton::soundTransform property exists");
 	} else {
 	    DejaGnu.fail("SimpleButton::soundTransform property doesn't exist");
 	}
	if (Std.is(x1.trackAsMenu, Bool)) {
	    DejaGnu.pass("SimpleButton::trackAsMenu property exists");
	} else {
	    DejaGnu.fail("SimpleButton::trackAsMenu property doesn't exist");
	}
 	if (Std.is(x1.upState, DisplayObject)) {
 	    DejaGnu.pass("SimpleButton::upState property exists");
 	} else {
 	    DejaGnu.fail("SimpleButton::upState property doesn't exist");
 	}
	if (Std.is(x1.useHandCursor, Bool)) {
	    DejaGnu.pass("SimpleButton::useHandCursor property exists");
	} else {
	    DejaGnu.fail("SimpleButton::useHandCursor property doesn't exist");
	}

// Tests to see if all the methods exist. All these do is test for
// existance of a method, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.
// 	if (x1.SimpleButton == DisplayObject) {
// 	    DejaGnu.pass("SimpleButton::SimpleButton() method exists");
// 	} else {
// 	    DejaGnu.fail("SimpleButton::SimpleButton() method doesn't exist");
// 	}

#else
    DejaGnu.note("SimpleButton did not exist in versions prior to SWF9");
#end
        // Call this after finishing all tests. It prints out the totals.
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

