#define APSTUDIO_READONLY_SYMBOLS
#include "afxres.h"
#undef APSTUDIO_READONLY_SYMBOLS

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 9,0,82,0
 PRODUCTVERSION 9,0,82,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "Free Software Foundation, Inc.\0"
// ProductName has to be the exact string "Shockwave Flash" as JS Flash
// detect scripts tend to test for navigator.plugins["Shockwave Flash"].
// c.f. http://www.adobe.com/support/flash/how/shock/javaplugs/javaplugs03.html
            VALUE "ProductName", "Shockwave Flash\0"
//            VALUE "ProductName", "Gnash Plugin for Mozilla\0"
            VALUE "ProductVersion", "9, 0, 82, 0\0"
            VALUE "FileVersion", "9, 0, 82, 0\0"
// FileDescription needs to include the string "Shockwave Flash X.Y rZ"
// as JS Flash detect scripts may use it to determine Flash player version.
            VALUE "FileDescription", "npgnash plugin for <a href=""http://www.gnu.org/software/gnash/"">Gnash</a>, the GNU SWF player. (Shockwave Flash 9.0 r82)\0"
            VALUE "LegalCopyright", "Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "MIMEType", "application/x-shockwave-flash\0"
            VALUE "FileExtents", "swf\0"
            VALUE "FileOpenName", "Gnash plugin (*.swf)\0"
            VALUE "InternalName", "npgnash\0"
            VALUE "OriginalFilename", "npgnash.dll\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END
