// vaapi_common.h: VA API common (internal) includes and definitions
// 
// Copyright (C) 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef GNASH_VAAPI_COMMON_H
#define GNASH_VAAPI_COMMON_H

#ifdef HAVE_CONFIG_H
#include "gnashconfig.h"
#endif

#include <va/va.h>
#if VA_CHECK_VERSION(0,34,0)
#include <va/va_compat.h>
#endif

#include <memory>
#include <stdint.h>
#include <cstdint>
#include "dsodefs.h"

#endif // GNASH_VAAPI_COMMON_H

// local Variables:
// mode: C++
// indent-tabs-mode: nil
// End:

