// Font_as.hx:  ActionScript 3 "Font" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090515 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.

#if flash9
import flash.text.Font;
import flash.display.MovieClip;
import flash.text.FontStyle;
import flash.text.FontType;
#else
//import flash.Font;
//import flash.MovieClip;
#end
import flash.Lib;
import Type;

// import our testing API
import DejaGnu;

// Class must be named with the _as suffix, as that's the same name as the file.
class Font_as {
    static function main() {

#if !flash9
		DejaGnu.note("The Font class does not exist in versions prior to flash9");
#else
		
        var x1:Font = new Font();

        // Make sure we actually get a valid class        
        if (x1 != null) {
            DejaGnu.pass("Font class exists");
        } else {
            DejaGnu.fail("Font class doesn't exist");
        }
		
// Tests to see if all the properties exist. All these do is test for
// existance of a property, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.

	//Need to get a font so that the properties arent't Null
	var fonts:Array<Dynamic> = Font.enumerateFonts(true);
	x1 = fonts[0];
	
	if (Std.is(x1.fontName, String)) {
	    DejaGnu.pass("Font.fontName property exists");
	} else {
	    DejaGnu.fail("Font.fontName property doesn't exist");
	}
	if (Std.is(x1.fontStyle, String)) {
	    DejaGnu.pass("Font.fontStyle property exists");
	} else {
	    DejaGnu.fail("Font.fontStyle property doesn't exist");
	}
	if (Std.is(x1.fontType, String)) {
	    DejaGnu.pass("Font.fontType property exists");
	} else {
	    DejaGnu.fail("Font.fontType property doesn't exist");
	}

// Tests to see if all the methods exist. All these do is test for
// existance of a method, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.

	//enumerateFonts is a static method
	if (Type.typeof(Font.enumerateFonts) == ValueType.TFunction) {
	    DejaGnu.pass("Font::enumerateFonts() method exists");
	} else {
	    DejaGnu.fail("Font::enumerateFonts() method doesn't exist");
	}
	if (Type.typeof(x1.hasGlyphs) == ValueType.TFunction) {
	    DejaGnu.pass("Font::hasGlyphs() method exists");
	} else {
	    DejaGnu.fail("Font::hasGlyphs() method doesn't exist");
	}
	//registerFont is a static method
	if (Type.typeof(Font.registerFont) == ValueType.TFunction) {
	    DejaGnu.pass("Font::registerFont() method exists");
	} else {
	    DejaGnu.fail("Font::registerFont() method doesn't exist");
	}

#end
        // Call this after finishing all tests. It prints out the totals.
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

