// IMEConversionMode_as3.hx:  ActionScript 3 "IMEConversionMode" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090514 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.

#if flash9
import flash.system.IMEConversionMode;
import flash.display.MovieClip;
#else
//import flash.IMEConversionMode;
//import flash.MovieClip;
#end
import flash.Lib;
import Type;
import Std;

// import our testing API
import DejaGnu;

// Class must be named with the _as3 suffix, as that's the same name as the file.
class IMEConversionMode_as {
    static function main() {
#if flash9  
      // Make sure we actually get a valid class        
//	DejaGnu.note("ME1 "+ Type.typeof(IMEConversionMode.ALPHANUMERIC_FULL) );

//Si:
//Check the following enumeration!


        if (Std.is(IMEConversionMode.ALPHANUMERIC_FULL , String) ) {
            DejaGnu.pass("IMEConversionMode.ALPHANUMERIC_FULL constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.ALPHANUMERIC_FULL constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.ALPHANUMERIC_HALF, String) )  {
            DejaGnu.pass("IMEConversionMode.ALPHANUMERIC_HALF constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.ALPHANUMERIC_HALF constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.CHINESE, String) )  {
            DejaGnu.pass("IMEConversionMode.CHINESE constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.CHINESE constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.JAPANESE_HIRAGANA	, String) )  {
            DejaGnu.pass("IMEConversionMode.JAPANESE_HIRAGANA constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.JAPANESE_HIRAGANA constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.JAPANESE_KATAKANA_FULL , String) )  {
            DejaGnu.pass("IMEConversionMode.JAPANESE_KATAKANA_FULL constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.JAPANESE_KATAKANA_FULL constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.JAPANESE_KATAKANA_HALF , String) )  {
            DejaGnu.pass("IMEConversionMode.JAPANESE_KATAKANA_HALF constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.JAPANESE_KATAKANA_HALF constant doesn't exist");
        }

        if (Std.is(IMEConversionMode.KOREAN , String) ) {
            DejaGnu.pass("IMEConversionMode.KOREAN constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.KOREAN constant doesn't exist");
        }
        if (Std.is(IMEConversionMode.UNKNOWN , String) )  {
            DejaGnu.pass("IMEConversionModeUNKNOWN. constant exists");
        } else {
            DejaGnu.fail("IMEConversionMode.UNKNOWN constant doesn't exist");
        }

        // Call this after finishing all tests. It prints out the totals.
#else
        DejaGnu.note("The IMEConversionMode class did not exist in versions prior to SWF9");
        
#end
        
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

