// ContextMenuItem_as.hx:  ActionScript 3 "ContextMenuItem" class, for Gnash.
//
// Generated by gen-as3.sh on: 20090515 by "rob". Remove this
// after any hand editing loosing changes.
//
//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// This test case must be processed by CPP before compiling to include the
//  DejaGnu.hx header file for the testing framework support.

#if flash9
import flash.ui.ContextMenuItem;
import flash.display.MovieClip;
#else
import flash.ContextMenuItem;
import flash.MovieClip;
import flash.ContextMenu;
#end
import flash.Lib;
import Type;
import Std;

// import our testing API
import DejaGnu;

// Class must be named with the _as suffix, as that's the same name as the file.
class ContextMenuItem_as {
    static function main() {
        #if flash9
		var x1:ContextMenuItem = new ContextMenuItem("Caption", false, true, true);

        // Make sure we actually get a valid class        
        if (x1 != null) {
            DejaGnu.pass("ContextMenuItem class exists");
        } else {
            DejaGnu.fail("ContextMenuItem class doesn't exist");
        }
// Tests to see if all the properties exist. All these do is test for
// existance of a property, and don't test the functionality at all. This
// is primarily useful only to test completeness of the API implementation.
	if (Std.is(x1.caption, String) && (Std.string(x1.caption) == "Caption")) {
	    DejaGnu.pass("ContextMenuItem.caption property exists");
	} else {
	    DejaGnu.fail("ContextMenuItem.caption property doesn't exist");
	}
	if (Type.typeof(x1.separatorBefore) == ValueType.TBool) {
	    DejaGnu.pass("ContextMenuItem.separatorBefore property exists");
	} else {
	    DejaGnu.fail("ContextMenuItem.separatorBefore property doesn't exist");
	}
	if (Type.typeof(x1.visible) == ValueType.TBool) {
	    DejaGnu.pass("ContextMenuItem.visible property exists");
	} else {
	    DejaGnu.fail("ContextMenuItem.visible property doesn't exist");
	}
//FIXME: test methods (not implemented in haxe)
//	if (Type.typeof(x1.clone) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::clone method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::clone method exists");
//	}
//	if (Type.typeof(x1.systemClearMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemClearMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemClearMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemCopyLinkMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemCopyLinkMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemCopyLinkMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemCopyMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemCopyMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemCopyMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemCutMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemCutMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemCutMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemOpenLinkMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemLinkMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemLinkMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemPasteMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemPasteMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemPasteMenuItem method exists");
//	}
//	if (Type.typeof(x1.systemSelectAllMenuItem) == ValueType.TFunction) {
//		DejaGnu.pass("ContextMenuItem::systemSelectAllMenuItem method exists");
//	} else {
//		DejaGnu.fail("ContextMenuItem::systemSelectAllMenuItem method exists");
//	}
	#end
	#if (flash7 || flash8)
		DejaGnu.note("Use the ContextMenu_as test to make sure this is working");
	#end
        // Call this after finishing all tests. It prints out the totals.
        DejaGnu.done();
    }
}

// local Variables:
// mode: C++
// indent-tabs-mode: t
// End:

